within ;
package Pump_Limited "Pump with maximum power"
  function calc_Q_ref
    import Modelica.Blocks.Math;
    extends Modelica.Icons.Function;
    input Real n;
    input Modelica.SIunits.Power P_max;
    input Real c[5];
    output Modelica.SIunits.VolumeFlowRate Q_ref;

  protected
    Real cubic_power[4];
    Real r[:,2];

  algorithm
    cubic_power :={c[1],c[2]*n,c[3]*n^2,c[4]*n^3+c[5]};
    r :=Modelica.Math.Vectors.Utilities.roots(cat(1,cubic_power[1:end - 1], {cubic_power[
  end] - P_max}));
    Q_ref := Pump_Limited.median(r[:,1]);
  end calc_Q_ref;

partial model Custom_Pump_Partial "Partial Model"
    import NonSI = Modelica.SIunits.Conversions.NonSIunits;
    import Modelica.Constants;

  extends Modelica.Fluid.Interfaces.PartialTwoPort(
    port_b_exposesState = energyDynamics<>Modelica.Fluid.Types.Dynamics.SteadyState
                                                                     or massDynamics<>Modelica.Fluid.Types.Dynamics.SteadyState,
    port_a(
      p(start=p_a_start),
      m_flow(start = m_flow_start,
             min = if allowFlowReversal and not checkValve then -Constants.inf else 0)),
    port_b(
      p(start=p_b_start),
      m_flow(start = -m_flow_start,
             max = if allowFlowReversal and not checkValve then +Constants.inf else 0)));

  // Initialization
  parameter Medium.AbsolutePressure p_a_start=system.p_start
      "Guess value for inlet pressure"
    annotation(Dialog(tab="Initialization"));
  parameter Medium.AbsolutePressure p_b_start=p_a_start
      "Guess value for outlet pressure"
    annotation(Dialog(tab="Initialization"));
  parameter Medium.MassFlowRate m_flow_start = system.m_flow_start
      "Guess value of m_flow = port_a.m_flow"
    annotation(Dialog(tab = "Initialization"));

  final parameter Modelica.SIunits.VolumeFlowRate V_flow_single_init=
      m_flow_start/rho_nominal
    "Used for simplified initialization model";
  final parameter Modelica.SIunits.Height delta_head_init=flowCharacteristic(
      V_flow_single_init,1) - flowCharacteristic(0,1)
    "Used for simplified initialization model";

  // Characteristic curves

  replaceable function flowCharacteristic =
      Custom_Pump_V2.BaseClasses_Custom.PumpCharacteristics.quadraticFlow "Head vs. V_flow characteristic"
    annotation(Dialog(group="Characteristics"), choicesAllMatching=true);

  replaceable function powerCharacteristic =
        Custom_Pump_V2.BaseClasses_Custom.PumpCharacteristics.cubicPower
      "Power consumption vs. V_flow at nominal speed and density"
      annotation(Dialog(group="Characteristics"), choicesAllMatching=true);

 final parameter Medium.Density rho_nominal = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default);
   //   "Nominal fluid density for characteristic"
 //   annotation(Dialog(group="Characteristics"));

  // Assumptions
  parameter Boolean checkValve=false "= true to prevent reverse flow"
    annotation(Dialog(tab="Assumptions"), Evaluate=true);

  parameter Modelica.SIunits.Volume V=0 "Volume inside the pump"
    annotation (Dialog(tab="Assumptions"), Evaluate=true);

  replaceable parameter Modelica.SIunits.Power P_max=30;

  // Energy and mass balance
  extends Modelica.Fluid.Interfaces.PartialLumpedVolume(
      final fluidVolume = V,
      energyDynamics=Modelica.Fluid.Types.Dynamics.SteadyState,
      massDynamics=Modelica.Fluid.Types.Dynamics.SteadyState,
      final p_start = p_b_start);

  // Heat transfer through boundary, e.g., to add a housing
  parameter Boolean use_HeatTransfer = false
      "= true to use a HeatTransfer model, e.g., for a housing"
      annotation (Dialog(tab="Assumptions",group="Heat transfer"));
  replaceable model HeatTransfer =
      Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer
    constrainedby
      Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer
      "Wall heat transfer"
      annotation (Dialog(tab="Assumptions",group="Heat transfer",enable=use_HeatTransfer),choicesAllMatching=true);
  HeatTransfer heatTransfer(
    redeclare final package Medium = Medium,
    final n=1,
    surfaceAreas={4*Modelica.Constants.pi*(3/4*V/Modelica.Constants.pi)^(2/3)},
    final states = {medium.state},
    final use_k = use_HeatTransfer)
      annotation (Placement(transformation(
        extent={{-10,-10},{30,30}},
        rotation=180,
        origin={50,-10})));
  Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer
    annotation (Placement(transformation(extent={{30,-70},{50,-50}})));

  // Variables
  final parameter Modelica.SIunits.Acceleration g=system.g;
  Medium.Density rho = medium.d;
  Modelica.SIunits.Pressure dp_pump=port_b.p - port_a.p "Pressure increase";
  Modelica.SIunits.Height head=flowCharacteristic(60*60*V_flow_single, N) "Pump head";
  Modelica.SIunits.MassFlowRate m_flow=port_a.m_flow "Mass flow rate (total)";
  Modelica.SIunits.MassFlowRate m_flow_single=m_flow
    "Mass flow rate (single pump)";
  Modelica.SIunits.VolumeFlowRate V_flow "Volume flow rate (total)";
  Modelica.SIunits.VolumeFlowRate V_flow_single(start=m_flow_start/rho_nominal) "Volume flow rate (single pump)";
  Real N "Shaft rotational speed";
  Modelica.SIunits.Power W_single = powerCharacteristic(60*60*V_flow_single,N) "Power Consumption (single pump)";
  Modelica.SIunits.Power W_total=W_single "Power Consumption (total)";
  Real eta "Global Efficiency";
  final constant Medium.MassFlowRate unit_m_flow=1 annotation (HideResult=true);
  Real s(start = m_flow_start/unit_m_flow)
      "Curvilinear abscissa for the flow curve in parametric form (either mass flow rate or head)";

  // Diagnostics
  replaceable model Monitoring =
    Custom_Pump_V2.BaseClasses_Custom.PumpMonitoring.PumpMonitoringBase
      constrainedby
      Custom_Pump_V2.BaseClasses_Custom.PumpMonitoring.PumpMonitoringBase
      "Optional pump monitoring"
      annotation(Dialog(tab="Advanced", group="Diagnostics"), choicesAllMatching=true);
  final parameter Boolean show_NPSHa = false
      "obsolete -- remove modifier and specify Monitoring for NPSH instead"
    annotation(Dialog(tab="Advanced", group="Obsolete"));
  Monitoring monitoring(
          redeclare final package Medium = Medium,
          final state_in = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow)),
          final state = medium.state) "Monitoring model"
     annotation (Placement(transformation(extent={{-64,-42},{-20,0}})));
  protected
  constant Modelica.SIunits.Height unitHead=1;
  constant Modelica.SIunits.MassFlowRate unitMassFlowRate=1;

equation

  // Flow equations
   V_flow = homotopy(m_flow/rho,
                     m_flow/rho_nominal);
   V_flow_single = V_flow;

    // Regular flow characteristics without check valve
    head=dp_pump/(rho*g);

    s = 0;

    eta = dp_pump*V_flow_single/W_single;

    //W_single = min(P_max, powerCharacteristic(60*60*V_flow_single));

  // Energy balance
  Wb_flow = W_total;
  Qb_flow = heatTransfer.Q_flows[1];
  Hb_flow = port_a.m_flow*actualStream(port_a.h_outflow) +
            port_b.m_flow*actualStream(port_b.h_outflow);

  // Ports
  port_a.h_outflow = medium.h;
  port_b.h_outflow = medium.h;
  port_b.p = medium.p
      "outlet pressure is equal to medium pressure, which includes Wb_flow";

  // Mass balance
  mb_flow = port_a.m_flow + port_b.m_flow;

  mbXi_flow = port_a.m_flow*actualStream(port_a.Xi_outflow) +
              port_b.m_flow*actualStream(port_b.Xi_outflow);
  port_a.Xi_outflow = medium.Xi;
  port_b.Xi_outflow = medium.Xi;

  mbC_flow = port_a.m_flow*actualStream(port_a.C_outflow) +
             port_b.m_flow*actualStream(port_b.C_outflow);
  port_a.C_outflow = C;
  port_b.C_outflow = C;

  connect(heatTransfer.heatPorts[1], heatPort) annotation (Line(
      points={{40,-34},{40,-60}},
      color={127,0,0}));
  annotation (
    Icon(coordinateSystem(preserveAspectRatio=true,  extent={{-100,-100},{100,
              100}}), graphics={
          Rectangle(
            extent={{-100,46},{100,-46}},
            lineColor={0,0,0},
            fillColor={0,127,255},
            fillPattern=FillPattern.HorizontalCylinder),
          Polygon(
            points={{-48,-60},{-72,-100},{72,-100},{48,-60},{-48,-60}},
            lineColor={0,0,255},
            pattern=LinePattern.None,
            fillColor={0,0,0},
            fillPattern=FillPattern.VerticalCylinder),
          Ellipse(
            extent={{-80,80},{80,-80}},
            lineColor={0,0,0},
            fillPattern=FillPattern.Sphere,
            fillColor={0,100,199}),
          Polygon(
            points={{-28,30},{-28,-30},{50,-2},{-28,30}},
            lineColor={0,0,0},
            pattern=LinePattern.None,
            fillPattern=FillPattern.HorizontalCylinder,
            fillColor={255,255,255})}),
    Documentation(info="<html>
<p>This is the base model for pumps.
<p>The model describes a centrifugal pump, or a group of <code>nParallel</code> identical pumps. The pump model is based on the theory of kinematic similarity: the pump characteristics are given for nominal operating conditions (rotational speed and fluid density), and then adapted to actual operating condition, according to the similarity equations.

<p><b>Pump characteristics</b></p>
<p> The nominal hydraulic characteristic (head vs. volume flow rate) is given by the the replaceable function <code>flowCharacteristic</code>.
<p> The pump energy balance can be specified in two alternative ways:
<ul>
<li><code>use_powerCharacteristic = false</code> (default option): the replaceable function <code>efficiencyCharacteristic</code> (efficiency vs. volume flow rate in nominal conditions) is used to determine the efficiency, and then the power consumption.
    The default is a constant efficiency of 0.8.</li>
<li><code>use_powerCharacteristic = true</code>: the replaceable function <code>powerCharacteristic</code> (power consumption vs. volume flow rate in nominal conditions) is used to determine the power consumption, and then the efficiency.
    Use <code>powerCharacteristic</code> to specify a non-zero power consumption for zero flow rate.
</ul>
<p>
Several functions are provided in the package <code>PumpCharacteristics</code> to specify the characteristics as a function of some operating points at nominal conditions.
<p>Depending on the value of the <code>checkValve</code> parameter, the model either supports reverse flow conditions, or includes a built-in check valve to avoid flow reversal.
</p>
<p>It is possible to take into account the mass and energy storage of the fluid inside the pump by specifying its volume <code>V</code>, and by selecting appropriate dynamic mass and energy balance assumptions (see below);
this is recommended to avoid singularities in the computation of the outlet enthalpy in case of zero flow rate.
If zero flow rate conditions are always avoided, this dynamic effect can be neglected by leaving the default value <code>V = 0</code>, thus avoiding fast state variables in the model.
</p>

<p><b>Dynamics options</b></p>
<p>
Steady-state mass and energy balances are assumed per default, neglecting the holdup of fluid in the pump; this configuration works well if the flow rate is always positive.
Dynamic mass and energy balance can be used by setting the corresponding dynamic parameters. This is recommended to avoid singularities at zero or reversing mass flow rate. If the initial conditions imply non-zero mass flow rate, it is possible to use the <code>SteadyStateInitial</code> condition, otherwise it is recommended to use <code>FixedInitial</code> in order to avoid undetermined initial conditions.
</p>

<p><b>Heat transfer</b></p>
<p>
The Boolean parameter <code>use_HeatTransfer</code> can be set to true if heat exchanged with the environment
should be taken into account or to model a housing. This might be desirable if a pump with realistic
<code>powerCharacteristic</code> for zero flow operates while a valve prevents fluid flow.
</p>

<p><b>Diagnostics of Cavitation</b></p>
<p>The replaceable Monitoring submodel can be configured to PumpMonitoringNPSH,
in order to compute the Net Positive Suction Head available and check for cavitation,
provided a two-phase medium model is used (see Advanced tab).
</p>
</html>",
      revisions="<html>
<ul>
<li><i>8 Jan 2013</i>
    by R&uuml;diger Franke:<br>
    moved NPSH diagnostics from PartialPump to replaceable sub-model PumpMonitoring.PumpMonitoringNPSH (see ticket #646)</li>
<li><i>Dec 2008</i>
    by R&uuml;diger Franke:<br>
    <ul>
    <li>Replaced simplified mass and energy balances with rigorous formulation (base class PartialLumpedVolume)</li>
    <li>Introduced optional HeatTransfer model defining Qb_flow</li>
    <li>Enabled events when the checkValve is operating to support the opening of a discrete valve before port_a</li>
    </ul></li>
<li><i>31 Oct 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
end Custom_Pump_Partial;

model Pump_vs_lim
  "Centrifugal pump with variable speed and limited power"
  extends Pump_Limited.Custom_Pump_Partial(
    redeclare function flowCharacteristic =
        Pump_Limited.PumpCharacteristics.quadraticFlow (c=
            alpha),
    redeclare function powerCharacteristic =
        Pump_Limited.PumpCharacteristics.cubicPower (c=
            beta),
    W_single=if N_in_internal <=0 then 1e-3 else min(P_max, powerCharacteristic(60*60*V_flow_single, N_actual)),
    head=if N_in_internal<=0 or flowCharacteristic(60*60*V_flow_single, N_actual) <=0 then 0 else if W_single == P_max then flowCharacteristic(60*60*V_flow_single,Pump_Limited.calc_n_ref(
        60*60*V_flow_single,
        P_max,
        beta)) else flowCharacteristic(60*60*V_flow_single, N_actual));

 /*   head=if W_single == P_max then flowCharacteristic(Pump_Limited.calc_Q_ref(
        N,
        P_max,
        beta),N) else flowCharacteristic(60*60*V_flow_single, N)); */


        Real abc = Pump_Limited.calc_Q_ref(N_actual,P_max,beta);
        Real head_cmp = flowCharacteristic(60*60*V_flow_single, N_actual);
  parameter Boolean use_N_in = false
   "Get the relative rotational speed from the input connector";

   parameter Real rpm_rel=1
   "Rotational speed (relative)"
   annotation(Dialog(enable = not use_N_in));

   parameter Boolean filteredOpening=false "Rotational speed filtered by 2nd order critical damper" annotation(Dialog(enable=use_N_in));
   parameter Real riseTime=1 "Rise time of the filter (time to reach 99.6 % of an input step)" annotation(Dialog(enable=filteredOpening));

   Modelica.Blocks.Continuous.Filter filter(order=2, f_cut=5/(2*Modelica.Constants.pi
        *riseTime)) if filteredOpening;

Modelica.Blocks.Interfaces.RealInput N_in if use_N_in
    "Prescribed rotational speed"
    annotation (Placement(transformation(
        extent={{-20,-20},{20,20}},
        rotation=-90,
        origin={0,100}), iconTransformation(
        extent={{-20,-20},{20,20}},
        rotation=-90,
        origin={0,100})));

        parameter Real P_max = 100 "Maximum power of pump";
        parameter Real N_min = 1e-3 "Minimum relative rotational speed";
        parameter Real alpha[3] "Head vs. Volumetric flow coefficients" annotation(Dialog(group="Characteristics"));
        parameter Real beta[5] "Power vs. Volumetric flow coefficients" annotation(Dialog(group="Characteristics"));

  protected
  Modelica.Blocks.Interfaces.RealInput N_in_internal
    "Needed to connect to conditional connector";
   Modelica.Blocks.Interfaces.RealInput N_actual "Needed for filter";

equation
  // Connect statement active only if use_p_in = true
  connect(N_in, N_in_internal);

  // Internal connector value when use_p_in = false
  if not use_N_in then
    N_in_internal = rpm_rel;
  end if;
  // Set N with a lower limit to avoid singularities at zero speed and upper limit
  N = if N_in_internal<=0 then 1e-3 else min(max(N_in_internal,N_min),1) "Rotational speed";

  connect(N, filter.u);
  connect(filter.y, N_actual);

  if not filteredOpening then
     N_actual = N;
  end if;
  //W_single = min(powerCharacteristic(60*60*V_flow_single,N),P_max);

end Pump_vs_lim;

  function median
    extends Modelica.Icons.Function;
    input Real v[:]; //Input Vector
    output Real med;

  protected
    Real v_sorted[:];

  algorithm
    v_sorted := Modelica.Math.Vectors.sort(v);
    if mod(size(v,1),2) <> 0 then
      med := v_sorted[integer((size(v,1)+1)/2)];

    else
      med := 0.5*(v_sorted[integer(size(v,1)/2)]+v_sorted[integer(size(v,1)/2+1)]);

    end if;
  end median;

  function calc_n_ref
    import Modelica.Blocks.Math;
    extends Modelica.Icons.Function;
    input Modelica.SIunits.VolumeFlowRate Q;
    input Modelica.SIunits.Power P_max;
    input Real c[5];
    output Real n_ref;

  protected
    Real cubic_power[4];
    Real r[:,2];
    Real r_real[:];

  algorithm
    cubic_power :={c[4],c[3]*Q,c[2]*Q^2,c[1]*Q^3+c[5]};
    r :=Modelica.Math.Vectors.Utilities.roots(cat(1,cubic_power[1:end - 1], {cubic_power[
  end] - P_max}));
    for ii in 1:size(r[:,2],1) loop
      if r[ii,2] ==0 then
        r_real := cat(1,r_real, {r[ii,1]});
      end if;
    end for;
    //n_ref := Pump_Limited.median(r_real);
    n_ref := max(r_real);
  end calc_n_ref;

model Pump_vs_lim_2
    "Centrifugal pump with variable speed (limited by N_min) and limited power. RPM=0 means that pump is turned off"
  extends Pump_Limited.Custom_Pump_Partial(
    redeclare function flowCharacteristic =
        Pump_Limited.PumpCharacteristics.quadraticFlow (c=
            alpha),
    redeclare function powerCharacteristic =
        Pump_Limited.PumpCharacteristics.cubicPower (c=
            beta),
    W_single=if N_in_internal <=0 then 1e-3 else min(P_max, powerCharacteristic(60*60*V_flow_single, N)),
    head=if N_in_internal<=0 or flowCharacteristic(60*60*V_flow_single, N) <=0 then 0 else if W_single == P_max then flowCharacteristic(60*60*V_flow_single,Pump_Limited.calc_n_ref(
        60*60*V_flow_single,
        P_max,
        beta)) else flowCharacteristic(60*60*V_flow_single, N));

 /*   head=if W_single == P_max then flowCharacteristic(Pump_Limited.calc_Q_ref(
        N,
        P_max,
        beta),N) else flowCharacteristic(60*60*V_flow_single, N)); */

        Real abc = Pump_Limited.calc_Q_ref(N,P_max,beta);
        Real head_cmp = flowCharacteristic(60*60*V_flow_single, N);
  parameter Boolean use_N_in = false
   "Get the relative rotational speed from the input connector";

   parameter Real rpm_rel=1
   "Rotational speed (relative)"
   annotation(Dialog(enable = not use_N_in));

Modelica.Blocks.Interfaces.RealInput N_in if use_N_in
    "Prescribed rotational speed"
    annotation (Placement(transformation(
        extent={{-20,-20},{20,20}},
        rotation=-90,
        origin={0,100}), iconTransformation(
        extent={{-20,-20},{20,20}},
        rotation=-90,
        origin={0,100})));

        parameter Real P_max = 100 "Maximum power of pump";
        parameter Real N_min = 1e-3 "Minimum relative rotational speed";
        parameter Real alpha[3] "Head vs. Volumetric flow coefficients" annotation(Dialog(group="Characteristics"));
        parameter Real beta[5] "Power vs. Volumetric flow coefficients" annotation(Dialog(group="Characteristics"));

  protected
  Modelica.Blocks.Interfaces.RealInput N_in_internal
    "Needed to connect to conditional connector";
equation
  // Connect statement active only if use_p_in = true
  connect(N_in, N_in_internal);
  // Internal connector value when use_p_in = false
  if not use_N_in then
    N_in_internal = rpm_rel;
  end if;
  // Set N with a lower limit to avoid singularities at zero speed and upper limit
  N = if N_in_internal<=0 then 1e-3 else min(max(N_in_internal,N_min),1) "Rotational speed";

  //W_single = min(powerCharacteristic(60*60*V_flow_single,N),P_max);

end Pump_vs_lim_2;

model Pump_vs_lim_filtered
    "Centrifugal pump with variable speed (limited by N_min, can be filtered) and limited power, RPM=0 means that pump is turned off"
  extends Pump_Limited.Custom_Pump_Partial(
    redeclare function flowCharacteristic =
        Pump_Limited.PumpCharacteristics.quadraticFlow (c=
            alpha),
    redeclare function powerCharacteristic =
        Pump_Limited.PumpCharacteristics.cubicPower (c=
            beta),
    W_single=if N_actual <=0 then 1e-3 else min(P_max, powerCharacteristic(60*60*V_flow_single, N_actual)),
    head=if N_actual<=0 or flowCharacteristic(60*60*V_flow_single, N_actual) <=0 then 0 else if W_single == P_max then flowCharacteristic(60*60*V_flow_single,Pump_Limited.calc_n_ref(
        60*60*V_flow_single,
        P_max,
        beta)) else flowCharacteristic(60*60*V_flow_single, N_actual));

 /*   head=if W_single == P_max then flowCharacteristic(Pump_Limited.calc_Q_ref(
        N,
        P_max,
        beta),N) else flowCharacteristic(60*60*V_flow_single, N)); */

        Real abc = Pump_Limited.calc_Q_ref(N_actual,P_max,beta);
        Real head_cmp = flowCharacteristic(60*60*V_flow_single, N_actual);
  parameter Boolean use_N_in = false
   "Get the relative rotational speed from the input connector";

   parameter Real rpm_rel=1
   "Rotational speed (relative)"
   annotation(Dialog(enable = not use_N_in));

   parameter Boolean filteredOpening=false "Rotational speed filtered by 2nd order critical damper" annotation(Dialog(enable=use_N_in));
   parameter Real riseTime=1 "Rise time of the filter (time to reach 99.6 % of an input step)" annotation(Dialog(enable=filteredOpening));

   Modelica.Blocks.Continuous.Filter filter(order=2, f_cut=5/(2*Modelica.Constants.pi
        *riseTime), u=N) if filteredOpening;

Modelica.Blocks.Interfaces.RealInput N_in if use_N_in
    "Prescribed rotational speed"
    annotation (Placement(transformation(
        extent={{-20,-20},{20,20}},
        rotation=-90,
        origin={0,100}), iconTransformation(
        extent={{-20,-20},{20,20}},
        rotation=-90,
        origin={0,100})));

        parameter Real P_max = 100 "Maximum power of pump";
        parameter Real N_min = 1e-3 "Minimum relative rotational speed";
        parameter Real alpha[3] "Head vs. Volumetric flow coefficients" annotation(Dialog(group="Characteristics"));
        parameter Real beta[5] "Power vs. Volumetric flow coefficients" annotation(Dialog(group="Characteristics"));

       Modelica.Blocks.Interfaces.RealOutput N_actual "Needed for filter";

  protected
  Modelica.Blocks.Interfaces.RealInput N_in_internal
    "Needed to connect to conditional connector";



equation
  // Connect statement active only if use_p_in = true
  connect(N_in, N_in_internal);

  // Internal connector value when use_p_in = false
  if not use_N_in then
    N_in_internal = rpm_rel;
  end if;
  // Set N with a lower limit to avoid singularities at zero speed and upper limit
  N = if N_in_internal<=0 then 0 else min(max(N_in_internal,N_min),1) "Rotational speed";

  connect(filter.y, N_actual);

  if not filteredOpening then
     N_actual = N;
  end if;
  //W_single = min(powerCharacteristic(60*60*V_flow_single,N),P_max);

end Pump_vs_lim_filtered;
  annotation (uses(Modelica(version="3.2.3")));
end Pump_Limited;
